package de.kebidge.camunda;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.http.HttpClient.Redirect;
import java.net.http.HttpClient.Version;
import java.net.http.HttpResponse.BodyHandlers;
import java.time.Duration;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;

public class GlobalQuoteDelegate implements JavaDelegate {

    private final static Logger LOGGER = Logger.getLogger("de.kebidge.camunda.GlobalQuoteDelegate");

    // execute ServiceTask_2 (Get Price and Change)
    public void execute(DelegateExecution execution) throws Exception {

        // process variables
        String symbol_process = "symbol";
        String symbol_value = (String) execution.getVariable(symbol_process);
        String quote_value = "";
        String price_process = "price";
        String change_process = "change";

        /*
         *  dummy data for user task 2
         *  **************************
         *      -> for usage comment out line 125 and line 126
         */
        double price_value = 0.123;
        double change_value = -0.987;

        // set your alphavantage key here
        // https://www.alphavantage.co/support/#api-key
        String key = "VPNHOQPE0AOJ8BEP";

        LOGGER.info("Processing request by '" + symbol_value + "'...");

        // build GET Request
        // GET https://www.alphavantage.co/query?function=GLOBAL_QUOTE&symbol=IBM&apikey=VPNHOQPE0AOJ8BEP
        String apiURL = "https://www.alphavantage.co/";
        String queryEndpoint = "query?function=GLOBAL_QUOTE&symbol=";
        String apiKey = "&apikey=" + key;

        String uri_string = apiURL + queryEndpoint + symbol_value + apiKey;

        // set HTTP Client
        HttpClient vClient = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(20))
                .followRedirects(Redirect.NORMAL).version(Version.HTTP_1_1).build();

        // set HTTP Request
        HttpRequest vRequest = HttpRequest.newBuilder().headers("Content-Type", "application/json")
                .timeout(Duration.ofSeconds(20))
                .uri(URI.create(uri_string)).build();

        LOGGER.info("Build GET Request: " + uri_string);

        try {
            HttpResponse<String> vResponse = vClient.send(vRequest, BodyHandlers.ofString());

            // check requested resource is available
            if (vResponse.statusCode() == HttpServletResponse.SC_NOT_FOUND) {
                
                // requested resource is not available (404)
                // set null to process variables
                execution.setVariable(price_process, null);
                execution.setVariable(change_process, null);                
                LOGGER.info("SC_NOT_FOUND -> set variable price and change to null");
            
            } else {

                // requested resource is available (200)

                // get quote resonse from REST API -> save to quote_value
                quote_value = vResponse.body();

                /* parse GLOBAL_QUOTE JSON 

                Example:
                
                {
                    "Global Quote": {
                        "01. symbol": "IBM",
                        "02. open": "116.4900",
                        "03. high": "116.5600",
                        "04. low": "115.2700",
                        "05. price": "116.0500",
                        "06. volume": "5384548",
                        "07. latest trading day": "2021-11-19",
                        "08. previous close": "116.6600",
                        "09. change": "-0.6100",
                        "10. change percent": "-0.5229%"
                    }
                }
                */
                
                // parse JSON object
                JsonParser jsonParser = new JsonParser();                
                JsonElement jsonTree = jsonParser.parse(quote_value);

                if(jsonTree.isJsonObject()) {
                    JsonObject jsonObject = jsonTree.getAsJsonObject();

                    // value = get(key)
                    JsonElement quote_element = jsonObject.get("Global Quote");

                    if(quote_element.isJsonObject()){
                        JsonObject quoteObject = quote_element.getAsJsonObject();

                        // check symbol item is not null
                        if(quoteObject.get("01. symbol") != null) {

                            // real data for user task 2
                            // comment out these variables while using dummy data 
                            price_value = quoteObject.get("05. price").getAsDouble();
                            change_value = quoteObject.get("09. change").getAsDouble();

                            LOGGER.info("JSON Parser: price=" + price_value);
                            LOGGER.info("JSON Parser: change=" + change_value);

                            // set price value and change value to process variables
                            execution.setVariable(price_process, price_value);
                            execution.setVariable(change_process, change_value);

                            LOGGER.info("HttpServletResponse succeed...");
                            LOGGER.info("Set process variable " + price_process + " = " + price_value);
                            LOGGER.info("Set process variable " + change_process + " = " + change_value);

                        } else {
                            
                            // symbol item is null
                            LOGGER.info("JSON is empty...");
                            LOGGER.info(quote_value);
                        }                        
                    }

                } else {

                    // quote_value is not a JSON object
                    LOGGER.info("JSON Parser failed...");
                    LOGGER.info("Not a JSON Object");
                    LOGGER.info(quote_value);
                }
            }
        } catch (IOException | InterruptedException e) {

            // IOException
            // set null to process variables
            execution.setVariable(price_process, null);
            execution.setVariable(change_process, null);
            LOGGER.info("CATCH IOException -> set variables price and change to null");
        }
    }
}