package de.kebidge.camunda;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.net.http.HttpClient.Redirect;
import java.net.http.HttpClient.Version;
import java.net.http.HttpResponse.BodyHandlers;
import java.time.Duration;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonArray;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;

public class SymbolSearchDelegate implements JavaDelegate {

    private final static Logger LOGGER = Logger.getLogger("de.kebidge.camunda.SymbolDelegate");

    // execute ServiceTask_1 (Get Symbol)
    public void execute(DelegateExecution execution) throws Exception {

        // process variables
        String stock_process = "stock";
        String stock_value = (String) execution.getVariable(stock_process);
        String symbol_process = "symbol";
        String symbol_value = "";
        String name_process = "name";
        String name_value = "";

        // set alphavantage key here
        // https://www.alphavantage.co/support/#api-key
        String key = "VPNHOQPE0AOJ8BEP";

        LOGGER.info("Processing request by '" + stock_value + "'...");

        // build GET Request
        // GET https://www.alphavantage.co/query?function=SYMBOL_SEARCH&keywords=IBM&apikey=VPNHOQPE0AOJ8BEP
        String apiURL = "https://www.alphavantage.co/";
        String queryEndpoint = "query?function=SYMBOL_SEARCH&keywords=";
        String apiKey = "&apikey=" + key;

        String uri_string = apiURL + queryEndpoint + stock_value + apiKey;

        // set HTTP Client
        HttpClient vClient = HttpClient.newBuilder().connectTimeout(Duration.ofSeconds(20))
                .followRedirects(Redirect.NORMAL).version(Version.HTTP_1_1).build();

        // set HTTP Request
        HttpRequest vRequest = HttpRequest.newBuilder().headers("Content-Type", "application/json")
                .timeout(Duration.ofSeconds(20))
                .uri(URI.create(uri_string)).build();

        LOGGER.info("Build GET Request: " + uri_string);

        try {
            HttpResponse<String> vResponse = vClient.send(vRequest, BodyHandlers.ofString());

            // check requested resource is available
            if (vResponse.statusCode() == HttpServletResponse.SC_NOT_FOUND) {
                
                // requested resource is not available (404)
                // set null to process variable
                execution.setVariable(symbol_process, null);                
                LOGGER.info("SC_NOT_FOUND -> set variable symbol to null");
            } else {
                
                // requested resource is available (200)

                // get symbol resonse from REST API -> save to best matches string
                String best_matches = vResponse.body();

                /* parse SYMBOL_SEARCH JSON 

                Example:


                {
                    "bestMatches": [
                        {
                            "1. symbol": "IBM",
                            "2. name": "International Business Machines Corp",
                            "3. type": "Equity",
                            "4. region": "United States",
                            "5. marketOpen": "09:30",
                            "6. marketClose": "16:00",
                            "7. timezone": "UTC-04",
                            "8. currency": "USD",
                            "9. matchScore": "1.0000"
                        },
                        { 
                            ...
                        }
                    }                      
                */
                
                // parse JSON object
                JsonParser jsonParser = new JsonParser();                
                JsonElement jsonTree = jsonParser.parse(best_matches);

                if(jsonTree.isJsonObject()) {
                    
                    JsonObject jsonObject = jsonTree.getAsJsonObject();
                    //LOGGER.info("jsonObject=" + jsonObject);

                    JsonElement best_maches_element = jsonObject.get("bestMatches");
                    //LOGGER.info("best_maches_element=" + best_maches_element);

                    if(best_maches_element.isJsonArray()){

                        // get json object (bestMatches) as json array
                        JsonArray symbol_array = best_maches_element.getAsJsonArray();
                        //LOGGER.info("JSON Parser: symbol_array=" + symbol_array);
                        
                        // get first array item as json object
                        JsonObject symbol_json_object = symbol_array.get(0).getAsJsonObject();
                        //LOGGER.info("JSON Parser: symbol_json_object=" + symbol_json_object);

                        // get symbol value from REST API
                        symbol_value = symbol_json_object.get("1. symbol").getAsString();
                        LOGGER.info("JSON Parser: symbol=" + symbol_value);

                        // get company name value from REST API
                        name_value = symbol_json_object.get("2. name").getAsString();
                        LOGGER.info("JSON Parser: name=" + name_value);

                        // set symbol value to process variable
                        execution.setVariable(symbol_process, symbol_value);

                        // set company name value to process variable
                        execution.setVariable(name_process, name_value);

                        // log success and symbol value
                        LOGGER.info("HttpServletResponse succeed...");
                    } else {
                            
                        // not a json array
                        LOGGER.info("NOT a JSON Array...");
                        LOGGER.info(best_maches_element.toString());
                    }
                } else {

                    // quote_value is not a JSON object
                    LOGGER.info("JSON Parser failed...");
                    LOGGER.info("Not a JSON Object");
                    LOGGER.info(best_matches);
                }
            }
        } catch (IOException | InterruptedException e) {
                
            // IOException
            // set null to process variable
            execution.setVariable(symbol_process, null);
            LOGGER.info("CATCH IOException -> set variable symbol to null");
        }
    }
}